"""Wrapper for bkr cmdline client calls."""

import subprocess
import time
import typing


class Bkr:
    """Wrapper for bkr cmdline client calls."""

    username: typing.ClassVar[str | None] = None
    password: typing.ClassVar[str | None] = None

    def __init__(self, args: list[str], *, timeout_s: int = 600) -> None:
        """Initialize object with command line arguments."""
        self.args = ["bkr"]
        self.args.append(args[0])
        if self.username and self.password:
            self.args.extend([f"--username={self.username}", f"--password={self.password}"])
        self.args.extend(args[1:])
        self.timeout_s = timeout_s

    def run_once(self) -> subprocess.CompletedProcess[str]:
        """Run a bkr command and return the completed process.

        On timeout, changes the return code to 127.
        """
        try:
            return subprocess.run(
                self.args,
                encoding="utf8",
                capture_output=True,
                timeout=self.timeout_s,
                check=False,
            )
        except subprocess.TimeoutExpired as e:
            return subprocess.CompletedProcess(
                self.args,
                127,
                e.stdout.decode("utf8") if e.stdout else "",
                e.stderr.decode("utf8") if e.stderr else "",
            )

    def run_repeatedly(
        self,
        *,
        max_attempts: int = 5,
        pause_s: int = 60,
    ) -> subprocess.CompletedProcess[str]:
        """Run command until it exits with 0 or we run out of retries.

        Returns the last completed process.

        On timeout, changes the return code to 127.
        """
        for i in range(max_attempts):
            time.sleep(i * pause_s)
            p = self.run_once()
            if p.returncode == 0:
                break
            print(f"bkr command {self.args} failed (try #{i + 1}):")
            print(p.stderr)
        return p
